@@ -10,11 +10,12 @@ import pandas as pd
 import pyproj
 import rasterio as rio
 from rasterio.features import geometry_mask
+from scipy.spatial import KDTree
 from shapely.geometry import MultiPoint, MultiPolygon, Point, Polygon
 from sklearn.base import BaseEstimator
 
-from elapid.stats import get_raster_stats_methods
-from elapid.types import CRSType, to_iterable
+from elapid.stats import get_raster_stats_methods, normalize_sample_probabilities
+from elapid.types import CRSType, Vector, to_iterable
 from elapid.utils import (
     NoDataException,
     check_raster_alignment,
@@ -55,6 +56,48 @@ def xy_to_geoseries(
     return gs
 
 
+def stack_geometries(
+    presence: Vector, background: Vector, add_class_label: bool = False, target_crs: str = "presence"
+) -> gpd.GeoDataFrame:
+    """Concatenate geometries from two GeoSeries/GeoDataFrames.
+
+    Args:
+        presence: presence geometry (y=1) locations
+        background: background geometry (y=0) locations
+        add_class_label: add a column labeling the y value for each point
+        target_crs: if reprojection is necessary, use this variable's crs.
+            valid options are "presence" and "background"
+
+    Returns:
+        merged GeoDataFrame with all geometries projected to the same crs.
+    """
+    pgeo = presence.geometry
+    bgeo = background.geometry
+
+    crs = presence.crs
+    if not crs_match(presence.crs, background.crs):
+        if target_crs.lower() == "presence":
+            bgeo = bgeo.to_crs(crs)
+        elif target_crs.lower() == "background":
+            crs = background.crs
+            pgeo = pgeo.to_crs(crs)
+        else:
+            raise NameError(f"Unrecognized target_crs option: {target_crs}")
+
+    attributes = {}
+    if add_class_label:
+        npresence = len(pgeo)
+        nbackground = len(bgeo)
+        classes = np.zeros(npresence + nbackground, dtype="uint8")
+        classes[:npresence] = 1
+        attributes["class"] = classes
+
+    geometry = pd.concat((pgeo, bgeo), axis=0, ignore_index=True)
+    gdf = gpd.GeoDataFrame(attributes, geometry=geometry, crs=crs)
+
+    return gdf
+
+
 def sample_raster(raster_path: str, count: int, ignore_mask: bool = False) -> gpd.GeoSeries:
     """Creates a random geographic sampling of points based on a raster's extent.
 
@@ -105,14 +148,14 @@ def sample_bias_file(raster_path: str, count: int, ignore_mask: bool = False) ->
             data = src.read(1)
             rows, cols = np.where(data)
             values = data.flatten()
-            probabilities = (values - values.min()) / (values - values.min()).sum()
+            probabilities = normalize_sample_probabilities(values)
             samples = np.random.choice(len(rows), size=count, p=probabilities)
 
         else:
             data = src.read(1, masked=True)
             rows, cols = np.where(~data.mask)
             values = data[rows, cols]
-            probabilities = (values - values.min()) / (values - values.min()).sum()
+            probabilities = normalize_sample_probabilities(values)
             samples = np.random.choice(len(rows), size=count, p=probabilities)
 
         xy = np.zeros((count, 2))
@@ -264,7 +307,7 @@ def annotate(
     raster_paths: Union[str, list],
     labels: list = None,
     drop_na: bool = True,
-):
+) -> gpd.GeoDataFrame:
     """Read raster values for each point in a vector and append as new columns.
 
     Args:
@@ -277,7 +320,7 @@ def annotate(
         drop_na: drop all records with no-data values.
 
     Returns:
-        gdf: GeoDataFrame annotated with the pixel values from each raster
+        GeoDataFrame annotated with the pixel values from each raster
     """
     # format the inputs
     raster_paths = to_iterable(raster_paths)
@@ -360,7 +403,6 @@ def annotate_geoseries(
 
     # get the dataset dimensions
     n_rasters = len(raster_paths)
-    n_points = len(points)
 
     # create arrays and flags for updating
     raster_values = []
@@ -383,25 +425,20 @@ def annotate_geoseries(
             xys = [(point.x, point.y) for point in points]
 
             # read each pixel value
-            samples = src.sample(xys, masked=False)
-
-            # assign to an output array
-            outarr = np.zeros((n_points, src.count), dtype=dtype)
-            for idx, sample in enumerate(samples):
-                outarr[idx] = sample
+            samples = np.array(list(src.sample(xys, masked=False)), dtype=dtype)
 
             # identify nodata points to remove later
             if drop_na and src.nodata is not None:
                 nodata_flag = True
-                valid_idxs.append(outarr[:, 0] != src.nodata)
+                valid_idxs.append(samples[:, 0] != src.nodata)
 
-            raster_values.append(outarr)
+            raster_values.append(samples)
 
     # merge the arrays from each raster
     values = np.concatenate(raster_values, axis=1, dtype=dtype)
 
     if nodata_flag:
-        valid = np.max(valid_idxs, axis=0)
+        valid = np.all(valid_idxs, axis=0)
         values = values[valid, :]
         points = points.iloc[valid]
         points.index = range(valid.sum())
@@ -585,7 +622,7 @@ def apply_model_to_rasters(
                 continue
 
 
-def validate_gpd(geo: Union[gpd.GeoSeries, gpd.GeoDataFrame]) -> None:
+def validate_gpd(geo: Vector) -> None:
     """Validates whether an input is a GeoDataFrame or a GeoSeries.
 
     Args:
@@ -598,7 +635,7 @@ def validate_gpd(geo: Union[gpd.GeoSeries, gpd.GeoDataFrame]) -> None:
         raise TypeError("Input must be a GeoDataFrame or GeoSeries")
 
 
-def validate_polygons(geometry: Union[gpd.GeoSeries, gpd.GeoDataFrame]) -> pd.Index:
+def validate_polygons(geometry: Vector) -> pd.Index:
     """Iterate over a geoseries to find rows with invalid geometry types.
 
     Args:
@@ -652,7 +689,7 @@ def read_raster_from_polygon(src: rio.DatasetReader, poly: Union[Polygon, MultiP
 
 
 def zonal_stats(
-    polygons: Union[gpd.GeoSeries, gpd.GeoDataFrame],
+    polygons: Vector,
     raster_paths: list,
     labels: list = None,
     all_touched: bool = True,
@@ -758,3 +795,78 @@ def zonal_stats(
         merged = gpd.GeoDataFrame(pd.concat(raster_dfs, axis=1), geometry=polygons, crs=polygons.crs)
 
     return merged
+
+
+# sample weighting methods
+
+
+def nearest_point_distance(
+    points1: Vector, points2: Vector = None, n_neighbors: int = 1, cpu_count: int = -1
+) -> np.ndarray:
+    """Compute the average euclidean distance to the nearest point in a series.
+
+    Args:
+        points1: return the closest distance *from* these points
+        points2: return the closest distance *to* these points
+            if None, compute the distance to the nearest points
+            in the points1 series
+        n_neighbors: compute the average distance to the nearest n_neighbors
+        cpu_count: number of cpus to use for estimation.
+            -1 uses all cores
+
+    Returns:
+        array of shape (len(points),) with the distance to
+            each point's nearest neighbor
+    """
+    if points1.crs.is_geographic:
+        warnings.warn("Computing distances using geographic coordinates is bad")
+
+    pta1 = np.array(list(zip(points1.geometry.x, points1.geometry.y)))
+    k_offset = 1
+
+    if points2 is None:
+        pta2 = pta1
+        k_offset += 1
+
+    else:
+        pta2 = np.array(list(zip(points2.geometry.x, points2.geometry.y)))
+        if not crs_match(points1.crs, points2.crs):
+            warnings.warn("CRS mismatch between points")
+
+    tree = KDTree(pta1)
+    k = np.arange(n_neighbors) + k_offset
+    distance, idx = tree.query(pta2, k=k, workers=cpu_count)
+
+    return distance.mean(axis=1)
+
+
+def distance_weights(points: Vector, n_neighbors: int = 1, center: str = "median", cpu_count: int = -1) -> np.ndarray:
+    """Compute sample weights based on the distance between points.
+
+    Assigns higher scores to isolated points, lower scores to clustered points.
+
+    Args:
+        points: point-format GeoSeries or GeoDataFrame
+        n_neighbors: compute the average distance to the nearest n_neighbors
+        center: rescale the weights to center the mean or median of the array on 1
+            accepts either 'mean' or 'median' as input.
+            pass None to ignore.
+        cpu_count: number of cpus to use for estimation.
+            -1 uses all cores
+
+    Returns:
+        array of shape (len(points),) with scaled sample weights. Scaling
+            is performed by dividing by the maximum value, preserving the
+            relative scale of differences between the min and max distance.
+    """
+    distances = nearest_point_distance(points, n_neighbors=n_neighbors, cpu_count=cpu_count)
+    weights = distances / distances.max()
+
+    if center is not None:
+        if center.lower() == "mean":
+            weights /= weights.mean()
+
+        elif center.lower() == "median":
+            weights /= np.median(weights)
+
+    return weights
