@@ -1,6 +1,7 @@
 """Custom maxent and typing data types."""
 from typing import Any, Union
 
+import geopandas as gpd
 import numpy as np
 import pandas as pd
 import pyproj
@@ -9,6 +10,7 @@ import pyproj
 Number = Union[int, float]
 ArrayLike = Union[np.ndarray, pd.DataFrame]
 CRSType = Union[pyproj.CRS, str]
+Vector = Union[gpd.GeoSeries, gpd.GeoDataFrame]
 
 
 # maxent feature transformations
